from setuptools import setup

setup(name='nanorm',
      py_modules=['nanorm', 'nanorm_test'],
      version='1.8.2',
      keywords='orm namo mini sample database sqlite nanorm nanoorm',
      description='A simple ORM framework for Python ( Nano ORM )',
      long_description='This is a streamlined Python ORM framework for SLQite3. It intended to use a file write ORM, provide some basic functions develop when small or micro projects.',
      author="Tao Jiayuan",
      author_email="taojy123@163.com",
      license="MIT",
      url="https://github.com/taojy123/nanorm",
      )

